/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.als;

import fr.amap.commons.util.IteratorWithException;
import fr.amap.lidar.amapvox.shot.Shot;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import javax.vecmath.Point3d;
import javax.vecmath.Vector3d;

/**
 *
 * @author pverley
 */
public class AlsShotReader implements IteratorWithException<Shot> {

    private final BufferedReader reader;
    private String line;
    private int shotId;
    private boolean hasNextCalled = false;
    private AlsShot shot;

    AlsShotReader(File shotsFile) throws FileNotFoundException, IOException {
        reader = new BufferedReader(new FileReader(shotsFile));
        //skip header
        reader.readLine();
        // initialise shot ID
        shotId = -1;
    }

    @Override
    public boolean hasNext() throws IOException {
        if (!hasNextCalled) {
            hasNextCalled = true;
            shot = nextShot();
        }
        return shot != null;
    }

    @Override
    public AlsShot next() throws IOException {
        
        if (hasNextCalled) {
            hasNextCalled = false;
            return shot;
        } else {
            return nextShot();
        }
    }

    private AlsShot nextShot() throws IOException {

        line = reader.readLine();
        if (null == line) {
            reader.close();
            return null;
        }

        // increment shot ID
        shotId++;

        String[] split = line.split(" ");
        double xOrigin = Double.valueOf(split[0]);
        double yOrigin = Double.valueOf(split[1]);
        double zOrigin = Double.valueOf(split[2]);

        double xDirection = Double.valueOf(split[3]);
        double yDirection = Double.valueOf(split[4]);
        double zDirection = Double.valueOf(split[5]);

        int nbEchos = Integer.valueOf(split[6]);

        double[] ranges = new double[nbEchos];
        int[] classifications = new int[nbEchos];

        for (int i = 0; i < ranges.length; i++) {
            if ((14 + i) > split.length) {
                throw new IOException("Columns missing inside shot file");
            }
            ranges[i] = Double.valueOf(split[7 + i]);
            classifications[i] = Integer.valueOf(split[14 + i]);
        }

        AlsShot shot = new AlsShot(shotId, new Point3d(xOrigin, yOrigin, zOrigin), new Vector3d(xDirection, yDirection, zDirection), ranges);
        shot.classifications = classifications;

        return shot;
    }
}
