/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package fr.amap.lidar.amapvox.voxelisation.tls;

import fr.amap.commons.math.matrix.Mat3D;
import fr.amap.commons.math.matrix.Mat4D;
import fr.amap.commons.math.util.MatrixUtility;
import fr.amap.lidar.amapvox.commons.VoxelSpaceInfos;
import fr.amap.lidar.amapvox.util.AVoxTask;
import fr.amap.lidar.amapvox.voxelisation.Voxelization;
import fr.amap.lidar.amapvox.voxelisation.configuration.TLSVoxCfg;
import fr.amap.lidar.amapvox.voxelisation.configuration.VoxelizationCfg;
import fr.amap.lidar.amapvox.voxelisation.configuration.params.VoxelParameters;
import java.io.File;
import org.apache.log4j.Logger;

/**
 *
 * @author calcul
 */
public abstract class TLSVoxelisation extends AVoxTask {

    private final static Logger LOGGER = Logger.getLogger(TLSVoxelisation.class);

    protected int nbVoxelisationFinished;
    protected File inputFile;
    protected Voxelization voxelAnalysis;
    protected Mat4D transfMatrix;
    protected Mat3D rotation;
    protected VoxelParameters parameters;
    protected File outputFile;
    protected final TLSVoxCfg cfg;

    public TLSVoxelisation(File file, int ncpu) {

        super(file, ncpu);
        cfg = new TLSVoxCfg();
    }

    public TLSVoxelisation(TLSVoxCfg cfg, int ncpu) {
        super(null, ncpu);
        this.cfg = cfg;
    }

    @Override
    public void init() throws Exception {

        if (null != file) {
            cfg.readConfiguration(file);
        }
        this.inputFile = cfg.getInputFile();
        this.parameters = cfg.getVoxelParameters();
        parameters.infos.setType(VoxelSpaceInfos.Type.TLS);
        this.outputFile = cfg.getOutputFile();

        nbVoxelisationFinished = 0;

        // Transformation matrices
        Mat4D pop = null != cfg.getPopMatrix()
                ? MatrixUtility.convertMatrix4dToMat4D(cfg.getPopMatrix())
                : Mat4D.identity();
        Mat4D vop = null != cfg.getVopMatrix()
                ? MatrixUtility.convertMatrix4dToMat4D(cfg.getVopMatrix())
                : Mat4D.identity();
        Mat4D sop = MatrixUtility.convertMatrix4dToMat4D(cfg.getSopMatrix());
        Mat4D popVop = Mat4D.multiply(pop, vop);
        transfMatrix = Mat4D.multiply(sop, popVop);

        rotation = new Mat3D();
        rotation.mat = new double[]{
            transfMatrix.mat[0], transfMatrix.mat[1], transfMatrix.mat[2],
            transfMatrix.mat[4], transfMatrix.mat[5], transfMatrix.mat[6],
            transfMatrix.mat[8], transfMatrix.mat[9], transfMatrix.mat[10]
        };

        voxelAnalysis = new Voxelization(cfg);
    }
    
    @Override
    public void setCancelled(boolean cancelled) {
        super.setCancelled(cancelled);
        // propagate cancellation to Voxelization.java
        voxelAnalysis.setCancelled(cancelled);
    }
}
